
--- Basic Global + Local Lighting
-- Charles Mallah, April 2018
-- Open license, do as you wish


local screenImage                  = nil
local width                        = nil
local height                       = nil
local canvas                       = nil
local LIGHT_SHADER                 = nil
local DARKNESS_SHADER              = nil
local lightOn                      = true
local lightImage                   = nil
local lightW                       = 0
local lightH                       = 0
local LOCAL_LIGHT_CANVAS           = nil
local LIGHTS                       = {}

local daylight                     = 0.25
local shadowC4                     = {0.5, 0.5, 0.5, 1}

function prepLight()
  love.graphics.setCanvas(LOCAL_LIGHT_CANVAS)
  love.graphics.clear(0, 0, 0, 0)
  for i = 1, #LIGHTS do
    local light = LIGHTS[i]
    love.graphics.setColor(light.c4[1], light.c4[2], light.c4[3], light.c4[4])
    love.graphics.draw(lightImage, light.x-light.w2, light.y-light.w2, 0, light.s, light.s)
  end
  love.graphics.setCanvas()
  
  love.graphics.setColor(1, 1, 1, 1)
end

function addLight(x, y, size)
  size                             = size or 700
  local scale                      = size / lightW
  
  local index                      = #LIGHTS + 1
  LIGHTS[index]                    = {
                                      x  = x,
                                      y  = y,
                                      w  = size,
                                      h  = size,
                                      w2 = size * 0.5,
                                      h2 = size * 0.5,
                                      s  = scale,
                                      c4 = {math.random(), math.random(), math.random(), 1}, -- Light colour, currently random
                                    }
  prepLight()
end

function wipeLights()
  LIGHTS = {}
  prepLight()
end

function testWhite()
  for i = 1, #LIGHTS do
    local light = LIGHTS[i]
    light.c4 = {0.8, 0.8, 0.5, 1}
  end
  prepLight()
end

function love.load()
  screenImage                      = love.graphics.newImage("screen.png")
  width                            = love.graphics:getWidth()
  height                           = love.graphics:getHeight()
  
  canvas                           = love.graphics.newCanvas(width, height)
  LOCAL_LIGHT_CANVAS               = love.graphics.newCanvas(width, height)
  
  lightImage                       = love.graphics.newImage("light.png")
  lightW                           = lightImage:getWidth()
  lightH                           = lightImage:getHeight()
  
  LIGHT_SHADER = love.graphics.newShader([[
  extern number daylight;
  vec4 effect(vec4 color, Image tex, vec2 tc, vec2 sc) {
    vec4 pixel    = Texel(tex, tc);
    pixel.rgb     *= (1.0 - daylight) * 0.05;
    pixel.rgb     = clamp(pixel.rgb, vec3(0.0, 0.0, 0.0), vec3(1.0, 1.0, 1.0));
    return pixel;
  }
  ]])
  
  DARKNESS_SHADER = love.graphics.newShader([[
  extern vec4 shadowRGB;
  vec4 effect(vec4 color, Image tex, vec2 tc, vec2 sc) {
    vec4 pixel    = Texel(tex, tc);
    if ( pixel.r == 0.0 && pixel.g == 0.0 && pixel.b == 0.0 ) {
      pixel.rgb   = shadowRGB.rgb;
    } else {
      pixel.rgb   += shadowRGB.rgb;
    }
    return pixel;
  }
  ]])
  
end

function love.update(dt)
  shadowC4[1]    = daylight
  shadowC4[2]    = daylight
  shadowC4[3]    = daylight
  DARKNESS_SHADER:send("shadowRGB", shadowC4)
  LIGHT_SHADER:send("daylight", daylight)
end

function love.draw()
	love.graphics.setCanvas(canvas) -- Here would be all your rendering pipe of the visible game area
  love.graphics.clear(0, 0, 0, 0)
	love.graphics.draw(screenImage) -- For now, background image
  love.graphics.setCanvas()
  
  love.graphics.draw(canvas) -- This can be drawn as is
  
	if lightOn then
    love.graphics.setBlendMode("multiply", "premultiplied") -- Darken with global lighting (where there are no lights)
    love.graphics.setShader(DARKNESS_SHADER)
    love.graphics.draw(LOCAL_LIGHT_CANVAS)
    love.graphics.setShader(LIGHT_SHADER) -- Lighting blended on top
    love.graphics.setBlendMode("add")
    love.graphics.draw(LOCAL_LIGHT_CANVAS)
    
    love.graphics.setBlendMode("alpha")
    love.graphics.setShader()
	end
  
  local x = 10
  local y = 10
  love.graphics.print("daylight: "..daylight, x, y)
  local y = y + 15
  love.graphics.print("lights: "..#LIGHTS, x, y)
  local y = y + 25
  love.graphics.print("key 1 to 4: set daylight", x, y)
  local y = y + 15
  love.graphics.print("mouse 1 add a random colored light", x, y)
  local y = y + 15
  love.graphics.print("mouse 2 wipe lights", x, y)
  local y = y + 15
  love.graphics.print("key 9 set all lights to white", x, y)
  
end

function love.mousepressed(x, y, button)
  if button == 1 then
    addLight(x, y)
  elseif button == 2 then
    wipeLights()
  end
end

function love.keypressed(key)
	if key == "z" then
		lightOn = not lightOn
	end
  
  if key == "1" then
    daylight = 0
  elseif key == "2" then
    daylight = 0.25
  elseif key == "3" then
    daylight = 0.5
  elseif key == "4" then
    daylight = 1.0
  elseif key == "9" then
    testWhite()
  end
  
end
